--[[ 
****************************************************
* Historian add-on for World of Warcraft.
* 
* Collects various interesting but ultimately useless statistics about your WoW characters.
****************************************************
]]

local L = AceLibrary("AceLocale-2.2"):new("Historian");
local f = AceLibrary("HistorianFormatting-1.0");


-- *************************************
-- * Chat commands
-- *************************************
local HistorianOptions = { 
    type='group',
    args = {
        report = {
			type = 'execute',
			name = L["Report"],
			desc = L["Writes statistics about your character to the chat."],
			func = "Report",
        },
		showTooltip = {
			type = "toggle",
			name = L["Show tooltip on enemies"],
			desc = L["Displays various statistics when your mouse is over known enemies."],
			get = "IsShowingTooltip",
			set = "ToggleShowingTooltip",
		},
    },
}


-- *************************************
-- * Set up Ace
-- *************************************
Historian = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1");

Historian:RegisterChatCommand(L["Slash-Commands"], HistorianOptions);

Historian:RegisterDB("HistorianDB", "HistorianCharDB");


-- *************************************
-- * Set up defaults
-- *************************************
Historian:RegisterDefaults("profile", {
	showTooltip = true
} )

Historian:RegisterDefaults("char", {
	Loot = { 
		MostMoneyEver = 0,
		ItemsOfQuality = { 0, 0, 0, 0, 0, 0 },
	},
	Advancement = {
		Levels = { },
		RecipesLearned = 0,
		TotalDeaths = 0,
		QuestsCompleted = 0,
		Tradeskills = { },
	},
	Discoveries = {
		Areas = 0,
	},
	EnvironmentalDamage = {
		Drowning = {
			Times = 0,
			TotalDamage = 0,
		},
		Fatique = {
			Times = 0,
			TotalDamage = 0,
		},
		Falling = {
			Times = 0,
			TotalDamage = 0,
		},
		Fire = {
			Times = 0,
			TotalDamage = 0,
		},
		Lava = {
			Times = 0,
			TotalDamage = 0,
		},
		Slime = {
			Times = 0,
			TotalDamage = 0,
		},
	},
	Follow = { 
		Players = {
		},
		MostFollowed = {
			Count = 0,
			Name = "",
		}
	},
	Kills = {
		Favorite = {
			Count = 0,
			Name = "",
		},
		Mobs = {
		},
		Total = 0,
	},
	Consumption = {
		Food = 0,
		Drink = 0,
	},
	Jumps = 0,
} )


-- *************************************
-- * Properties
-- *************************************
function Historian:IsShowingTooltip()
	return self.db.profile.showTooltip;
end
 
function Historian:ToggleShowingTooltip()
	self.db.profile.showTooltip = not self.db.profile.showTooltip;
end


-- *************************************
-- * OnInitialize
-- * Fires when the addon is first loaded
-- *************************************
function Historian:OnInitialize()
	self:MigrateData();	

	self.isEating = false;
	self.isDrinking = false;
end;


-- *************************************
-- * MigrateData
-- * Imports data stored in old formats
-- *************************************
function Historian:MigrateData()
	-- Migrate old data
	if (HISTORIAN_DATABASE ~= nil) then
		self:Print(L["Importing old data..."]);
		
		-- Loot stats
		self.db.char.Loot.MostMoneyEver = f:AsNumber(HISTORIAN_DATABASE["LootStats"]["MostMoneyEver"]);
		
		for i = 1, 6, 1 do
			self.db.char.Loot.ItemsOfQuality[i] = f:AsNumber(HISTORIAN_DATABASE["LootStats"]["LootedItemsOfQuality" .. tostring(i)]);
		end;
		
		-- Levels
		if (HISTORIAN_DATABASE["AdvancementStats"] ~= nil) then
			for i = 1, 70, 1 do
				local key = "Level_" .. tostring(i);

				if (HISTORIAN_DATABASE["AdvancementStats"][key] ~= nil) then
					self.db.char.Advancement.Levels[i] = HISTORIAN_DATABASE["AdvancementStats"][key];
				end;
			end;
		end;
		
		-- Total deaths
		self.db.char.Advancement.TotalDeaths = f:AsNumber(HISTORIAN_DATABASE["DeathStats"]["Deaths"]);
		
		-- Quests completed
		self.db.char.Advancement.QuestsCompleted= f:AsNumber(HISTORIAN_DATABASE["QuestStats"]["QuestsCompleted"]);
		
		-- Other advancements
		self.db.char.Advancement.RecipesLearned = f:AsNumber(HISTORIAN_DATABASE["AdvancementStats"]["RecipesLearned"]);
		
		-- Discoveries
		self.db.char.Discoveries.Areas = f:AsNumber(HISTORIAN_DATABASE["AdvancementStats"]["AreasDiscovered"]);
		
		-- Environmental damage
		self.db.char.EnvironmentalDamage.Drowning.Times = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageDrowningCount"]);
		self.db.char.EnvironmentalDamage.Drowning.TotalDamage = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageDrowningDamage"]);

		self.db.char.EnvironmentalDamage.Fatique.Times = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageFatiqueCount"]);
		self.db.char.EnvironmentalDamage.Fatique.TotalDamage = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageFatiqueDamage"]);

		self.db.char.EnvironmentalDamage.Falling.Times = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageFallingCount"]);
		self.db.char.EnvironmentalDamage.Falling.TotalDamage = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageFallingDamage"]);
		
		self.db.char.EnvironmentalDamage.Fire.Times = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageFireCount"]);
		self.db.char.EnvironmentalDamage.Fire.TotalDamage = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageFireDamage"]);
		
		self.db.char.EnvironmentalDamage.Lava.Times = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageLavaCount"]);
		self.db.char.EnvironmentalDamage.Lava.TotalDamage = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageLavaDamage"]);
		
		self.db.char.EnvironmentalDamage.Slime.Times = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageSlimeCount"]);
		self.db.char.EnvironmentalDamage.Slime.TotalDamage = f:AsNumber(HISTORIAN_DATABASE["DamageStats"]["EnvironmentDamageSlimeDamage"]);
		
		-- Follow
		self.db.char.Follow.MostFollowed.Count = f:AsNumber(HISTORIAN_DATABASE["MiscStats"]["MostFollowedCount"]);
		self.db.char.Follow.MostFollowed.Name = HISTORIAN_DATABASE["MiscStats"]["MostFollowedPlayer"];

		for key, value in pairs(HISTORIAN_DATABASE["MiscStats"]) do
			if (key == "MostFollowedCount") then
				self.db.char.Follow.MostFollowed.Count = f:AsNumber(value);
			else 
				if (key == "MostFollowedPlayer") then
					self.db.char.Follow.MostFollowed.Name = value;
				else
					local playerName = string.sub(key, 15);
					self.db.char.Follow.Players[playerName] = f:AsNumber(value);
				end;
			end;
		end;

		-- Kills
		for key, value in pairs(HISTORIAN_DATABASE["KillStats"]) do
			if (key == "_FavoritePreyCount") then
				self.db.char.Kills.Favorite.Count = f:AsNumber(value);
			else 
				if (key == "_FavoritePrey") then
					self.db.char.Kills.Favorite.Name = value;
				else
					if (key == "_MobsKilled") then
						self.db.char.Kills.Total = f:AsNumber(value);
					else
						self.db.char.Kills.Mobs[key] = f:AsNumber(value);
					end;
				end;
			end;
		end;

		

		self:Print(L["Import complete."]);
		HISTORIAN_DATABASE = nil;
	end;
end;


-- *************************************
-- * OnEnable
-- * Fires when the addon is enabled
-- *************************************
function Historian:OnEnable()
	-- Register for events
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	self:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");
	self:RegisterEvent("PLAYER_DEAD");
	self:RegisterEvent("PLAYER_LEVEL_UP");
	self:RegisterEvent("PLAYER_MONEY");
	self:RegisterEvent("AUTOFOLLOW_BEGIN");
	self:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS");
	self:RegisterEvent("LOOT_SLOT_CLEARED");
	self:RegisterEvent("CHAT_MSG_SYSTEM");
	self:RegisterEvent("LOOT_OPENED");
	self:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	self:RegisterEvent("TRADE_SKILL_SHOW");
	self:RegisterEvent("UNIT_AURA");
	
	self:SecureHook("AscendStop");
end;



-- *************************************
-- * Tooltip
-- *************************************
function Historian:UPDATE_MOUSEOVER_UNIT()
	if GameTooltip:IsVisible() and GameTooltipTextLeft1:GetText() ~= nil then
		if (not self.db.profile.showTooltip) then
			return;
		end;

		if (UnitIsPlayer("mouseover")) or (not UnitCreatureType("mouseover")) or (UnitIsFriend("player","mouseover")) then 
			return;
		end;

		local mobName = UnitName("mouseover");

		if (mobName ~= nil) then
			local numberOfKills = f:AsNumber(self.db.char.Kills.Mobs[mobName]);

			if (numberOfKills > 0) then
				local red = 1;
				local green = 1;
				local blue = 0;

				if (numberOfKills == 1) then
					GameTooltip:AddLine(L["Killed 1 time."], red, green, blue);
				else
					GameTooltip:AddLine(string.format(L["Killed %s times."], f:IntToStr(numberOfKills)), red, green, blue);
				end;
				GameTooltip:Show();
			end;
		end;
	end
end;



-- *************************************
-- * Report
-- *************************************
function Historian:Report()
	self:Print(L["----------- Historian Report -----------"]);
	self:Print(L["FORTUNE"]);
	self:Print(string.format(L["Money at peak: %s copper."], f:IntToStr(self.db.char.Loot.MostMoneyEver)));
	self:Print(L["--"]);
	
	self:Print(L["ITEMS LOOTED"]);
	self:Print(string.format(L["Common: %s"], f:IntToStr(self.db.char.Loot.ItemsOfQuality[1])));
	self:Print(string.format(L["Uncommon: %s"], f:IntToStr(self.db.char.Loot.ItemsOfQuality[2])));
	self:Print(string.format(L["Rare: %s"], f:IntToStr(self.db.char.Loot.ItemsOfQuality[3])));
	self:Print(string.format(L["Epic: %s"], f:IntToStr(self.db.char.Loot.ItemsOfQuality[4])));

	if (f:AsNumber(self.db.char.Loot.ItemsOfQuality[5]) > 0) then
		self:Print(string.format(L["Legendary: %s"], f:IntToStr(self.db.char.Loot.ItemsOfQuality[5])));
	end;

	if (f:AsNumber(self.db.char.Loot.ItemsOfQuality[6]) > 0) then
		self:Print(string.format(L["Artifact: %s"], f:IntToStr(self.db.char.Loot.ItemsOfQuality[6])));
	end;

	self:Print(L["--"]);
	
	self:Print(L["ACHIEVEMENTS"]);
	self:Print(string.format(L["Quests completed: %s"], f:IntToStr(self.db.char.Advancement.QuestsCompleted)));
	self:Print(string.format(L["Recipes learned: %s"], f:IntToStr(self.db.char.Advancement.RecipesLearned)));
	self:Print(string.format(L["Areas discovered: %s"], f:IntToStr(self.db.char.Discoveries.Areas)));
	self:Print(string.format(L["Total mobs killed: %s"], f:IntToStr(self.db.char.Kills.Total)));
	self:Print(L["--"]);

	self:Print(L["DISAPPOINTMENTS"]);
	if (f:AsNumber(self.db.char.EnvironmentalDamage.Drowning.Times) > 0) then
		self:Print(string.format(L["Drowning: %s times (%s damage)"], f:IntToStr(self.db.char.EnvironmentalDamage.Drowning.Times), f:IntToStr(self.db.char.EnvironmentalDamage.Drowning.TotalDamage)));
	end;
	if (f:AsNumber(self.db.char.EnvironmentalDamage.Fatique.Times) > 0) then
		self:Print(string.format(L["Fatiqued: %s times (%s damage)"], f:IntToStr(self.db.char.EnvironmentalDamage.Fatique.Times), f:IntToStr(self.db.char.EnvironmentalDamage.Fatique.TotalDamage)));
	end;
	if (f:AsNumber(self.db.char.EnvironmentalDamage.Falling.Times) > 0) then
		self:Print(string.format(L["Fallen down: %s times (%s damage)"], f:IntToStr(self.db.char.EnvironmentalDamage.Falling.Times), f:IntToStr(self.db.char.EnvironmentalDamage.Falling.TotalDamage)));
	end;
	if (f:AsNumber(self.db.char.EnvironmentalDamage.Fire.Times) > 0) then
		self:Print(string.format(L["Burnt: %s times (%s damage)"], f:IntToStr(self.db.char.EnvironmentalDamage.Fire.Times), f:IntToStr(self.db.char.EnvironmentalDamage.Fire.TotalDamage)));
	end;
	if (f:AsNumber(self.db.char.EnvironmentalDamage.Lava.Times) > 0) then
		self:Print(string.format(L["Tasted lava: %s times (%s damage)"], f:IntToStr(self.db.char.EnvironmentalDamage.Lava.Times), f:IntToStr(self.db.char.EnvironmentalDamage.Lava.TotalDamage)));
	end;
	if (f:AsNumber(self.db.char.EnvironmentalDamage.Slime.Times) > 0) then
		self:Print(string.format(L["Bathed in slime: %s times (%s damage)"], f:IntToStr(self.db.char.EnvironmentalDamage.Slime.Times), f:IntToStr(self.db.char.EnvironmentalDamage.Slime.TotalDamage)));
	end;
	self:Print(string.format(L["Just plain dead:: %s times."], f:IntToStr(f:AsNumber(self.db.char.Advancement.TotalDeaths))));
	self:Print(L["--"]);
	
	self:Print(L["HABITS"]);
	
	if (f:AsNumber(self.db.char.Follow.MostFollowed.Count) > 0) then
		self:Print(string.format(L["Most followed player: %s (%s times)"], self.db.char.Follow.MostFollowed.Name, f:IntToStr(self.db.char.Follow.MostFollowed.Count)));
	end;
	
	if (f:AsNumber(self.db.char.Kills.Favorite.Count) > 0) then
		self:Print(string.format(L["Favorite prey: %s (%s killed)"], self.db.char.Kills.Favorite.Name, f:IntToStr(self.db.char.Kills.Favorite.Count)));
	end;	

	if (f:AsNumber(self.db.char.Consumption.Food) > 0) then
		self:Print(string.format(L["You have eaten %s times."], f:IntToStr(self.db.char.Consumption.Food)));
	end;	

	if (f:AsNumber(self.db.char.Consumption.Drink) > 0) then
		self:Print(string.format(L["You have had a drink %s times."], f:IntToStr(self.db.char.Consumption.Drink)));
	end;	

	if (f:AsNumber(self.db.char.Jumps) > 0) then
		self:Print(string.format(L["Jumps: %s"], f:IntToStr(self.db.char.Jumps)));
	end;	

	self:Print(L["--"]);
	self:Print(L["NOTE: You probably need to scroll up to see the entire report. Get FuBar to see this in a nice GUI :)"]);
end;


-- *************************************
-- * GetFormatString
-- * Transforms a Blizzard chat message to a LUA regular expression
-- *************************************
function Historian:GetFormatString(localizedMessage)
	local formatted, replaced, replacedDig;

	formatted, replaced = string.gsub(localizedMessage, "%%s", "(.+)");
	formatted, replacedDig = string.gsub(formatted, "%%d", "(%%d%+)");

	return formatted;
end;


-- *************************************
-- * GetValuesFromChatMessage
-- * Returns the values stored in 'message' based on the Blizzard constant 'localizedMessage'.
-- * For example:
-- * 	local mobKilled = self:GetValuesFromChatMessage(chatMessage, SELFKILLOTHER);
-- * where chatMessage contains 'You have slain SomeEnemy!', returns 'SomeEnemy'.
-- *************************************
function Historian:GetValuesFromChatMessage(message, localizedMessage)
	local formatted = self:GetFormatString(localizedMessage);
	local v1,v2,v3,v4,v5,v6,v7,v8,v9;
	local result = {};

	-- TODO: Find a better way then declarating unknown return values
	for v1,v2,v3,v4,v5,v6,v7,v8,v9 in string.gmatch(message, formatted) do
		if (v1 ~= nil) then table.insert(result, v1); end;
		if (v2 ~= nil) then table.insert(result, v2); end;
		if (v3 ~= nil) then table.insert(result, v3); end;
		if (v4 ~= nil) then table.insert(result, v4); end;
		if (v5 ~= nil) then table.insert(result, v5); end;
		if (v6 ~= nil) then table.insert(result, v6); end;
		if (v7 ~= nil) then table.insert(result, v7); end;
		if (v8 ~= nil) then table.insert(result, v8); end;
		if (v9 ~= nil) then table.insert(result, v9); end;
	end;

	return unpack(result);
end;


-- *************************************
-- * IsWoWMessage
-- * Returns true if 'message' is of the type 'constant' 
-- * ('constant' contains placeholders so a straight comparison would not work)
-- *************************************
function Historian:IsWoWMessage(message, constant)
	return (string.find(message, self:GetFormatString(constant)) == 1);
end;



-- *************************************
-- * PLAYER_MONEY (Remember the biggest fortune the player ever had at any point in time)
-- *************************************
function Historian:PLAYER_MONEY()
	local highestAmount = self.db.char.Loot.MostMoneyEver;
	local currentMoney = GetMoney();

	if (currentMoney > highestAmount) then
		self.db.char.Loot.MostMoneyEver = currentMoney;
		self:TriggerEvent("Historian_Updated");
	end;
end;


-- *************************************
-- * CHAT_MSG_COMBAT_HOSTILE_DEATH (Player killed something)
-- *************************************
function Historian:CHAT_MSG_COMBAT_HOSTILE_DEATH(chatMessage)
	self.KillingBlowRegistered = true;
	self:OnKill(chatMessage);
end;


-- *************************************
-- * OnKill (Player killed something)
-- *************************************
function Historian:OnKill(chatMessage)
	local mobKilled = self:GetValuesFromChatMessage(chatMessage, SELFKILLOTHER);
		
	if (mobKilled == nil) then
		mobKilled = self:GetValuesFromChatMessage(chatMessage, COMBATLOG_XPGAIN_FIRSTPERSON);
	end;
		
	if (mobKilled ~= nil) then
		local killStatsForMob = f:AsNumber(self.db.char.Kills.Mobs[mobKilled]) + 1;
		self.db.char.Kills.Mobs[mobKilled] = killStatsForMob;

		local favoritePreyKills = self.db.char.Kills.Favorite.Count;
		
		if (killStatsForMob > favoritePreyKills) then
			self.db.char.Kills.Favorite.Count = killStatsForMob;

			local favoritePreyName = self.db.char.Kills.Favorite.Name;

			if (favoritePreyName ~= mobKilled) then
				self.db.char.Kills.Favorite.Name = mobKilled;
			end;
		end;
		
		local totalKills = f:AsNumber(self.db.char.Kills.Total) + 1;
		self.db.char.Kills.Total = totalKills;
		self:TriggerEvent("Historian_Updated");
	end;
end;


-- *************************************
-- * CHAT_MSG_COMBAT_XP_GAIN (Player got experience from killing something)
-- *************************************
function Historian:CHAT_MSG_COMBAT_XP_GAIN(chatMessage)
	if (self.KillingBlowRegistered == true) then
		self.KillingBlowRegistered = false;
		return;
	else
		return self:OnKill(chatMessage);
	end;
end;


-- *************************************
-- * CHAT_MSG_COMBAT_SELF_HITS (Player inflicted damage on himself)
-- *************************************
function Historian:CHAT_MSG_COMBAT_SELF_HITS(chatMessage)
	if (self:IsWoWMessage(chatMessage, VSENVIRONMENTALDAMAGE_DROWNING_SELF)) then
		return self:OnDrowning(chatMessage);
	end;

	if (self:IsWoWMessage(chatMessage, VSENVIRONMENTALDAMAGE_FATIGUE_SELF)) then
		return self:OnFatique(chatMessage);
	end;

	if (self:IsWoWMessage(chatMessage, VSENVIRONMENTALDAMAGE_FALLING_SELF)) then
		return self:OnFalling(chatMessage);
	end;

	if (self:IsWoWMessage(chatMessage, VSENVIRONMENTALDAMAGE_FIRE_SELF)) then
		return self:OnFireDamage(chatMessage);
	end;

	if (self:IsWoWMessage(chatMessage, VSENVIRONMENTALDAMAGE_LAVA_SELF)) then
		return self:OnLavaSwim(chatMessage);
	end;

	if (self:IsWoWMessage(chatMessage, VSENVIRONMENTALDAMAGE_SLIME_SELF)) then
		return self:OnSlimeSwim(chatMessage);
	end;
end;


-- *************************************
-- * PLAYER_DEAD (Player was killed)
-- *************************************
function Historian:PLAYER_DEAD()
	local totalDeaths = f:AsNumber(self.db.char.Advancement.TotalDeaths) + 1;

	self.db.char.Advancement.TotalDeaths = totalDeaths;

	self:TriggerEvent("Historian_Updated");
end;


-- *************************************
-- * PLAYER_LEVEL_UP (Player gained another level)
-- *************************************
function Historian:PLAYER_LEVEL_UP(newLevel)
	local levelInfo = { };
	levelInfo["Zone"] = GetRealZoneText(); 
	levelInfo["SubZone"] = GetSubZoneText();
	levelInfo["Date"] = date();
	
	self.db.char.Advancement.Levels[newLevel] = levelInfo;
	
	self:TriggerEvent("Historian_Updated");
end;


-- *************************************
-- * LOOT_OPENED (Loot dialog opened - used to prepare for LOOT_SLOT_CLEARED)
-- *************************************
function Historian:LOOT_OPENED()
	self.LootTable = {};
		
	local itemCount = GetNumLootItems();
	local slotNumber = 1;

	for slotNumber = 1, itemCount do
		if (LootSlotIsItem(slotNumber)) then
			local texture, item, quantity, quality = GetLootSlotInfo(slotNumber);
			if (quality ~= nil) then
				quality = f:AsNumber(quality);

				if (quality > 0) then
					self.LootTable[slotNumber] = quality;
				end;
			end;
		end;			
	end;
end;
	
	
-- *************************************
-- * LOOT_SLOT_CLEARED (Player looted an item)
-- *************************************
function Historian:LOOT_SLOT_CLEARED(slotNumber)
	if (slotNumber ~= nil) then
		if (self.LootTable[slotNumber] ~= nil) then
			local quality = f:AsNumber(self.LootTable[slotNumber]);
			if (quality > 0) then
				local totalLooted = f:AsNumber(self.db.char.Loot.ItemsOfQuality[quality]) + 1;
				self.db.char.Loot.ItemsOfQuality[quality] = totalLooted;
				self:TriggerEvent("Historian_Updated");
			end;
		end;
	end;
end;


-- *************************************
-- * CHAT_MSG_SYSTEM (Misc. chat messages)
-- *************************************
function Historian:CHAT_MSG_SYSTEM(chatMessage)
	if (self:IsWoWMessage(chatMessage, ERR_QUEST_COMPLETE_S)) then
		return self:OnQuestCompleted();
	end;

	if (self:IsWoWMessage(chatMessage, ERR_LEARN_RECIPE_S)) then
		return self:OnRecipeLearned();
	end;

	if (self:IsWoWMessage(chatMessage, ERR_ZONE_EXPLORED)) then
		return self:OnDiscovery();
	end;

	if (self:IsWoWMessage(chatMessage, ERR_ZONE_EXPLORED_XP)) then
		return self:OnDiscovery();
	end;
end;


function Historian:AUTOFOLLOW_BEGIN(playerFollowed)
	return self:OnFollow(playerFollowed);
end;


-- *************************************
-- * OnFollow (Player started following another player)
-- *************************************
function Historian:OnFollow(playerFollowed)
	local times = f:AsNumber(self.db.char.Follow.Players[playerFollowed]) + 1;
	local currentMax = f:AsNumber(self.db.char.Follow.MostFollowed.Count);
	
	self.db.char.Follow.Players[playerFollowed] = times;

	if (times > currentMax) then
		self.db.char.Follow.MostFollowed.Name = playerFollowed;
		self.db.char.Follow.MostFollowed.Count = times;
		self:TriggerEvent("Historian_Updated");
	end;
end;

	
-- *************************************
-- * OnQuestCompleted (Player completed yet another quest :)
-- *************************************
function Historian:OnQuestCompleted()
	local questsCompleted = f:AsNumber(self.db.char.Advancement.QuestsCompleted) + 1;
	
	self.db.char.Advancement.QuestsCompleted = questsCompleted;
	
	self:TriggerEvent("Historian_Updated");
end;



-- *************************************
-- * OnRecipeLearned (Player learned a new recipe)
-- *************************************
function Historian:OnRecipeLearned()
	local totalKnown = f:AsNumber(self.db.char.Advancement.RecipesLearned) + 1;
	
	self.db.char.Advancement.RecipesLearned = totalKnown;
	
	self:TriggerEvent("Historian_Updated");
end;
	

-- *************************************
-- * OnDiscovery (Player discovered a new zone/area)
-- *************************************
function Historian:OnDiscovery()
	local totalDiscovered = f:AsNumber(self.db.char.Discoveries.Areas) + 1;
	
	self.db.char.Discoveries.Areas = totalDiscovered;
	
	self:TriggerEvent("Historian_Updated");
end;

	
-- *************************************
-- * Environmental damage
-- *************************************
function Historian:RegisterEnvironmentDamage(type, damage)
	local times = f:AsNumber(self.db.char.EnvironmentalDamage[type].Times) + 1;
	local totalDamage = f:AsNumber(self.db.char.EnvironmentalDamage[type].TotalDamage) + f:AsNumber(damage);
	
	self.db.char.EnvironmentalDamage[type].Times = times;
	self.db.char.EnvironmentalDamage[type].TotalDamage = totalDamage;

	self:TriggerEvent("Historian_Updated");
end;
	
	
function Historian:OnDrowning(chatMessage)
	local damage = self:GetValuesFromChatMessage(chatMessage, VSENVIRONMENTALDAMAGE_DROWNING_SELF);
	if (damage ~= nil) then 
		self:RegisterEnvironmentDamage("Drowning", damage);
	end;
end;
	
function Historian:OnFatique(chatMessage)
	local damage = self:GetValuesFromChatMessage(chatMessage, VSENVIRONMENTALDAMAGE_FATIGUE_SELF);
	if (damage ~= nil) then 
		self:RegisterEnvironmentDamage("Fatique", damage);
	end;
end;
	
function Historian:OnFalling(chatMessage)
	local damage = self:GetValuesFromChatMessage(chatMessage, VSENVIRONMENTALDAMAGE_FALLING_SELF);
	if (damage ~= nil) then 
		self:RegisterEnvironmentDamage("Falling", damage);
	end;
end;
	
function Historian:OnFireDamage(chatMessage)
	local damage = self:GetValuesFromChatMessage(chatMessage, VSENVIRONMENTALDAMAGE_FIRE_SELF);
	if (damage ~= nil) then 
		self:RegisterEnvironmentDamage("Fire", damage);
	end;
end;
	
function Historian:OnLavaSwim(chatMessage)
	local damage = self:GetValuesFromChatMessage(chatMessage, VSENVIRONMENTALDAMAGE_LAVA_SELF);
	if (damage ~= nil) then 
		self:RegisterEnvironmentDamage("Lava", damage);
	end;
end;
	
function Historian:OnSlimeSwim(chatMessage)
	local damage = self:GetValuesFromChatMessage(chatMessage, VSENVIRONMENTALDAMAGE_SLIME_SELF);
	if (damage ~= nil) then 
		self:RegisterEnvironmentDamage("Slime", damage);
	end;
end;


-- *************************************
-- * TradeSkillShow
-- *************************************
function Historian:TRADE_SKILL_SHOW()
	if (self.db.char.Advancement.Tradeskills == nil) then
		self.db.char.Advancement.Tradeskills = {};
	end;
	
	local tradeSkillName = GetTradeSkillLine();
	local withHeaders = GetNumTradeSkills();
	
	if (self.db.char.Advancement.Tradeskills[tradeSkillName] == nil) then
		self.db.char.Advancement.Tradeskills[tradeSkillName] = {};
	else
		if (withHeaders == self.db.char.Advancement.Tradeskills[tradeSkillName].WithHeaders) then
			-- No updates required
			return;
		end;
	end;
	
	self.db.char.Advancement.Tradeskills[tradeSkillName].WithHeaders = withHeaders;
	
	-- Remove headers from the equation
	local total = 0;
	local index = 0;
	
	for index = GetFirstTradeSkill(), withHeaders, 1 do
		local skillName, skillType = GetTradeSkillInfo(index);
		if (skillName ~= nil) and (skillType ~= "header") then
			total = total + 1;
		end;
	end;
	
	
	self.db.char.Advancement.Tradeskills[tradeSkillName].Total = total;
	
	self:UpdateRecipeCount();
end;


-- *************************************
-- * UpdateRecipeCount
-- *************************************
function Historian:UpdateRecipeCount()
	local total = 0;
	
	for key, value in pairs(self.db.char.Advancement.Tradeskills) do
		total = total + f:AsNumber(value["Total"]);
	end;

	self.db.char.Advancement.RecipesLearned = total;
	self:TriggerEvent("Historian_Updated");
end;


-- *************************************
-- * UNIT_AURA
-- *************************************
function Historian:UNIT_AURA(unit)
	local wasEating = self.isEating;
	local wasDrinking = self.isDrinking;
	
	self.isEating = false;
	self.isDrinking = false;
	
	if (UnitIsUnit("player", unit)) then
		local i = 1;
		while UnitBuff("player", i) do
			buffName, _, icon, _, duration, timeleft = UnitBuff("player", i);
			
			if (buffName == L["Food"]) then
				self.isEating = true;
				if (not wasEating) then
					self.db.char.Consumption.Food = self.db.char.Consumption.Food + 1;
					self:TriggerEvent("Historian_Updated");
				end;
			else 
				if (buffName == L["Drink"]) then
					self.isDrinking = true;
					if (not wasDrinking) then
						self.db.char.Consumption.Drink = self.db.char.Consumption.Drink + 1;
						self:TriggerEvent("Historian_Updated");
					end;
				end;
			end;
			
			i = i + 1;
		end
	end;
end;


-- *************************************
-- * AscendStop
-- *************************************
function Historian:AscendStop()
	if (IsFlying() or IsSwimming()) then return; end;
	self.db.char.Jumps = self.db.char.Jumps + 1;
	self:TriggerEvent("Historian_Updated");
end;
