HistorianFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceEvent-2.0", "AceDB-2.0");
local L = AceLibrary("AceLocale-2.2"):new("HistorianFu");
local tablet = AceLibrary("Tablet-2.0");
local abacus = AceLibrary("Abacus-2.0");
local f = AceLibrary("HistorianFormatting-1.0");

HistorianFu:RegisterDB("HistorianFuDB");


-- *************************************
-- * Set up defaults / addon options
-- *************************************
HistorianFu.hasIcon = "Interface\\Icons\\INV_Scroll_05";
HistorianFu.hideWithoutStandby = true;
HistorianFu.independentProfile = false;
HistorianFu.tooltipHiddenWhenEmpty = true;


-- *************************************
-- * Properties
-- *************************************
function HistorianFu:IsShowingTooltip()
	return Historian:IsShowingTooltip();
end;
 
function HistorianFu:ToggleShowingTooltip()
	Historian:ToggleShowingTooltip();
	self:Update();
end;


-- *************************************
-- * Menu options
-- *************************************
HistorianFu.OnMenuRequest = {
	type = 'group',
	args = {
		showTooltip = {
			type = "toggle",
			name = L["Show tooltip on enemies"],
			desc = L["Displays certain statistics when your mouse is over known enemies."],
			get = "IsShowingTooltip",
			set = "ToggleShowingTooltip",
		},
	}
}
 

-- *************************************
-- * OnInitialize
-- *************************************
function HistorianFu:OnInitialize()
	self:RegisterEvent("Historian_Updated", "OnStatsUpdated");
end;
 
 
-- *************************************
-- * OnStatsUpdated
-- *************************************
function HistorianFu:OnStatsUpdated()
	self:Update();
end;


-- *************************************
-- * OnTooltipUpdate
-- *************************************
function HistorianFu:OnTooltipUpdate()

	-- *******************	
	-- FORTUNE
	-- *******************	

	local moneyCategory = tablet:AddCategory(
		'text', L["Fortune"],
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	
	moneyCategory:AddLine(
		'text', L["Money at peak:"],
		'text2', abacus:FormatMoneyFull(Historian.db.char.Loot.MostMoneyEver, true)
	)
	
	
	-- *******************	
	-- ITEMS LOOTED
	-- *******************	

	local lootCategory = tablet:AddCategory(
		'text', "Items looted",
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	
	local r,g,b = GetItemQualityColor(1);
      
	lootCategory:AddLine(
		'text', L["Common:"],
		'text2', f:IntToStr(Historian.db.char.Loot.ItemsOfQuality[1]),
		'textR', r,
		'textG', g,
    	'textB', b
	)
	
	r,g,b = GetItemQualityColor(2);
	
	lootCategory:AddLine(
		'text', L["Uncommon:"],
		'text2', f:IntToStr(Historian.db.char.Loot.ItemsOfQuality[2]),
		'textR', r,
		'textG', g,
    	'textB', b
	)
	
	r,g,b = GetItemQualityColor(3);
	
	lootCategory:AddLine(
		'text', L["Rare:"],
		'text2', f:IntToStr(Historian.db.char.Loot.ItemsOfQuality[3]),
		'textR', r,
		'textG', g,
    	'textB', b
	)
	
	r,g,b = GetItemQualityColor(4);
	
	lootCategory:AddLine(
		'text', L["Epic:"],
		'text2', f:IntToStr(Historian.db.char.Loot.ItemsOfQuality[4]),
		'textR', r,
		'textG', g,
    	'textB', b
	)

	r,g,b = GetItemQualityColor(5);

	if (Historian.db.char.Loot.ItemsOfQuality[5] > 0) then
		lootCategory:AddLine(
			'text', L["Legendary:"],
			'text2', f:IntToStr(Historian.db.char.Loot.ItemsOfQuality[5]),
			'textR', r,
			'textG', g,
			'textB', b
		)
	end;
	
	r,g,b = GetItemQualityColor(6);
	
	if (Historian.db.char.Loot.ItemsOfQuality[6] > 0) then
		lootCategory:AddLine(
			'text', L["Artifact:"],
			'text2', f:IntToStr(Historian.db.char.Loot.ItemsOfQuality[6]),
			'textR', r,
			'textG', g,
			'textB', b
		)
	end;	
	
	-- *******************	
	-- ACHIEVEMENTS
	-- *******************	

	local achievementsCategory = tablet:AddCategory(
		'text', L["Achievements"],
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	
	achievementsCategory:AddLine(
		'text', L["Quests completed:"],
		'text2', f:IntToStr(Historian.db.char.Advancement.QuestsCompleted)
	)
	
	achievementsCategory:AddLine(
		'text', L["Recipes learned:"],
		'text2', f:IntToStr(Historian.db.char.Advancement.RecipesLearned)
	)
	
	achievementsCategory:AddLine(
		'text', L["Areas discovered:"],
		'text2', f:IntToStr(Historian.db.char.Discoveries.Areas)
	)
	
	achievementsCategory:AddLine(
		'text', L["Total mobs killed:"],
		'text2', f:IntToStr(Historian.db.char.Kills.Total)
	)
	
	

	-- *******************	
	-- DISAPPOINTMENTS
	-- *******************	

	local disappointmentsCategory = tablet:AddCategory(
		'text', L["Disappointments"],
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)
	
	if (f:AsNumber(Historian.db.char.EnvironmentalDamage.Drowning.Times) > 0) then
		disappointmentsCategory:AddLine(
			'text', L["Drowning:"],
			'text2', string.format(L["%s times (%s damage)"], f:IntToStr(Historian.db.char.EnvironmentalDamage.Drowning.Times), f:IntToStr(Historian.db.char.EnvironmentalDamage.Drowning.TotalDamage))
		)
	end;
	
	if (f:AsNumber(Historian.db.char.EnvironmentalDamage.Fatique.Times) > 0) then
		disappointmentsCategory:AddLine(
			'text', "Fatiqued:",
			'text2', string.format(L["%s times (%s damage)"], f:IntToStr(Historian.db.char.EnvironmentalDamage.Fatique.Times), f:IntToStr(Historian.db.char.EnvironmentalDamage.Fatique.TotalDamage))
		)
	end;
	
	if (f:AsNumber(Historian.db.char.EnvironmentalDamage.Falling.Times) > 0) then
		disappointmentsCategory:AddLine(
			'text', "Fallen down:",
			'text2', string.format(L["%s times (%s damage)"], f:IntToStr(Historian.db.char.EnvironmentalDamage.Falling.Times), f:IntToStr(Historian.db.char.EnvironmentalDamage.Falling.TotalDamage))
		)
	end;
	
	if (f:AsNumber(Historian.db.char.EnvironmentalDamage.Fire.Times) > 0) then
		disappointmentsCategory:AddLine(
			'text', "Burnt:",
			'text2', string.format(L["%s times (%s damage)"], f:IntToStr(Historian.db.char.EnvironmentalDamage.Fire.Times), f:IntToStr(Historian.db.char.EnvironmentalDamage.Fire.TotalDamage))
		)
	end;
	
	if (f:AsNumber(Historian.db.char.EnvironmentalDamage.Lava.Times) > 0) then
		disappointmentsCategory:AddLine(
			'text', "Tasted lava:",
			'text2', string.format(L["%s times (%s damage)"], f:IntToStr(Historian.db.char.EnvironmentalDamage.Lava.Times), f:IntToStr(Historian.db.char.EnvironmentalDamage.Lava.TotalDamage))
		)
	end;
		
	if (f:AsNumber(Historian.db.char.EnvironmentalDamage.Slime.Times) > 0) then
		disappointmentsCategory:AddLine(
			'text', "Bathed in slime:",
			'text2', string.format(L["%s times (%s damage)"], f:IntToStr(Historian.db.char.EnvironmentalDamage.Slime.Times), f:IntToStr(Historian.db.char.EnvironmentalDamage.Slime.TotalDamage))
		)
	end;
	
	if (f:AsNumber(Historian.db.char.Advancement.TotalDeaths) > 0) then
		disappointmentsCategory:AddLine(
			'text', "Just plain dead:",
			'text2', string.format(L["%s times"], f:IntToStr(Historian.db.char.Advancement.TotalDeaths))
		)
	end;	
	
	-- *******************	
	-- HABITS
	-- *******************	

	local habitsCategory = tablet:AddCategory(
		'text', L["Habits"],
		'columns', 2,
		'child_textR', 1,
		'child_textG', 1,
		'child_textB', 0,
		'child_text2R', 1,
		'child_text2G', 1,
		'child_text2B', 1
	)

	if (Historian.db.char.Follow.MostFollowed.Count > 0) then
		habitsCategory:AddLine(
			'text', L["Most followed player:"],
			'text2', string.format(L["%s (%s times)"], Historian.db.char.Follow.MostFollowed.Name, f:IntToStr(Historian.db.char.Follow.MostFollowed.Count))
		)
	end;
	
	if (Historian.db.char.Kills.Favorite.Count > 0) then
		habitsCategory:AddLine(
			'text', L["Favorite prey:"],
			'text2', string.format(L["%s (%s killed)"], Historian.db.char.Kills.Favorite.Name, f:IntToStr(Historian.db.char.Kills.Favorite.Count))
		)
	end;

	habitsCategory:AddLine(
			'text', L["Had a snack:"],
			'text2', string.format(L["%s times"], f:IntToStr(Historian.db.char.Consumption.Food))
		)

	habitsCategory:AddLine(
			'text', L["Had a drink:"],
			'text2', string.format(L["%s times"], f:IntToStr(Historian.db.char.Consumption.Drink))
		)

	habitsCategory:AddLine(
			'text', L["Jumps:"],
			'text2', f:IntToStr(Historian.db.char.Jumps)
		)

end;